@@ -13,41 +13,38 @@ pub fn gen_function(def: &MethodDef, gen: &Gen) -> TokenStream {
 
     let abi_return_type = gen_win32_return_sig(&signature, gen);
 
-    let mut link = def
+    let link = def
         .impl_map()
         .expect("Function")
         .scope()
         .name()
         .to_lowercase();
 
-    let raw_dylib = cfg!(feature = "raw_dylib");
-
-    // TODO: remove this whole block once raw-dylib has stabilized as the workarounds
-    // will no longer be necessary.
-    if !raw_dylib && (link.contains("-ms-win-") || link == "d3dcompiler_47" || link == "sspicli") {
-        link = "onecoreuap".to_string();
-    }
-
     let link_attr = match def.static_lib() {
         Some(link) => quote! { #[link(name = #link, kind = "static")] },
         None => {
-            // TODO: switch to always using raw-dylib once it has stabilized
-            if raw_dylib {
-                quote! { #[link(name = #link, kind="raw-dylib")] }
-            } else {
-                quote! { #[link(name = #link)] }
-            }
+            quote! { #[link(name = #link)] }
         }
     };
 
+    let arch = if let Some(arch) = def.supported_arch() {
+        quote! { #[cfg(target_arch = #arch)] }
+    } else {
+        quote! {}
+    };
+
+    let features = method_features(&signature, gen);
+
     match signature.kind() {
         SignatureKind::Query => {
             let leading_params = &signature.params[..signature.params.len() - 2];
-            let args = leading_params.iter().map(|p| gen_win32_abi_arg(p));
+            let args = leading_params.iter().map(gen_win32_abi_arg);
             let params = gen_win32_params(leading_params, gen);
 
             quote! {
-                pub unsafe fn #name<#constraints T: ::windows::Interface>(#params) -> ::windows::Result<T> {
+                #arch
+                #features
+                pub unsafe fn #name<#constraints T: ::windows::runtime::Interface>(#params) -> ::windows::runtime::Result<T> {
                     #[cfg(windows)]
                     {
                         #link_attr
@@ -55,7 +52,7 @@ pub fn gen_function(def: &MethodDef, gen: &Gen) -> TokenStream {
                             fn #name(#(#abi_params),*) #abi_return_type;
                         }
                         let mut result__ = ::std::option::Option::None;
-                        #name(#(#args,)* &<T as ::windows::Interface>::IID, &mut result__ as *mut _ as *mut _).and_some(result__)
+                        #name(#(#args,)* &<T as ::windows::runtime::Interface>::IID, &mut result__ as *mut _ as *mut _).and_some(result__)
                     }
                     #[cfg(not(windows))]
                     unimplemented!("Unsupported target OS");
@@ -64,18 +61,20 @@ pub fn gen_function(def: &MethodDef, gen: &Gen) -> TokenStream {
         }
         SignatureKind::QueryOptional => {
             let leading_params = &signature.params[..signature.params.len() - 2];
-            let args = leading_params.iter().map(|p| gen_win32_abi_arg(p));
+            let args = leading_params.iter().map(gen_win32_abi_arg);
             let params = gen_win32_params(leading_params, gen);
 
             quote! {
-                pub unsafe fn #name<#constraints T: ::windows::Interface>(#params result__: *mut ::std::option::Option<T>) -> ::windows::Result<()> {
+                #arch
+                #features
+                pub unsafe fn #name<#constraints T: ::windows::runtime::Interface>(#params result__: *mut ::std::option::Option<T>) -> ::windows::runtime::Result<()> {
                     #[cfg(windows)]
                     {
                         #link_attr
                         extern "system" {
                             fn #name(#(#abi_params),*) #abi_return_type;
                         }
-                        #name(#(#args,)* &<T as ::windows::Interface>::IID, result__ as *mut _ as *mut _).ok()
+                        #name(#(#args,)* &<T as ::windows::runtime::Interface>::IID, result__ as *mut _ as *mut _).ok()
                     }
                     #[cfg(not(windows))]
                     unimplemented!("Unsupported target OS");
@@ -84,19 +83,21 @@ pub fn gen_function(def: &MethodDef, gen: &Gen) -> TokenStream {
         }
         SignatureKind::ResultValue => {
             let leading_params = &signature.params[..signature.params.len() - 1];
-            let args = leading_params.iter().map(|p| gen_win32_abi_arg(p));
+            let args = leading_params.iter().map(gen_win32_abi_arg);
             let params = gen_win32_params(leading_params, gen);
             let return_type_tokens = gen_win32_result_type(&signature, gen);
 
             quote! {
-                pub unsafe fn #name<#constraints>(#params) -> ::windows::Result<#return_type_tokens> {
+                #arch
+                #features
+                pub unsafe fn #name<#constraints>(#params) -> ::windows::runtime::Result<#return_type_tokens> {
                     #[cfg(windows)]
                     {
                         #link_attr
                         extern "system" {
                             fn #name(#(#abi_params),*) #abi_return_type;
                         }
-                        let mut result__: <#return_type_tokens as ::windows::Abi>::Abi = ::std::mem::zeroed();
+                        let mut result__: <#return_type_tokens as ::windows::runtime::Abi>::Abi = ::std::mem::zeroed();
                         #name(#(#args,)* &mut result__).from_abi::<#return_type_tokens>(result__)
                     }
                     #[cfg(not(windows))]
@@ -106,10 +107,12 @@ pub fn gen_function(def: &MethodDef, gen: &Gen) -> TokenStream {
         }
         SignatureKind::ResultVoid => {
             let params = gen_win32_params(&signature.params, gen);
-            let args = signature.params.iter().map(|p| gen_win32_abi_arg(p));
+            let args = signature.params.iter().map(gen_win32_abi_arg);
 
             quote! {
-                pub unsafe fn #name<#constraints>(#params) -> ::windows::Result<()> {
+                #arch
+                #features
+                pub unsafe fn #name<#constraints>(#params) -> ::windows::runtime::Result<()> {
                     #[cfg(windows)]
                     {
                         #link_attr
@@ -125,9 +128,11 @@ pub fn gen_function(def: &MethodDef, gen: &Gen) -> TokenStream {
         }
         SignatureKind::ReturnStruct | SignatureKind::PreserveSig => {
             let params = gen_win32_params(&signature.params, gen);
-            let args = signature.params.iter().map(|p| gen_win32_abi_arg(p));
+            let args = signature.params.iter().map(gen_win32_abi_arg);
 
             quote! {
+                #arch
+                #features
                 pub unsafe fn #name<#constraints>(#params) #abi_return_type {
                     #[cfg(windows)]
                     {
