@@ -14,7 +14,7 @@ pub fn gen_winrt_upcall(sig: &MethodSignature, inner: TokenStream, gen: &Gen) ->
                         let (ok_data__, ok_data_len__) = ok__.into_abi();
                         *result__ = ok_data__;
                         *result_size__ = ok_data_len__;
-                        ::windows::HRESULT(0)
+                        ::windows::runtime::HRESULT(0)
                     }
                     ::std::result::Result::Err(err) => err.into()
                 }
@@ -26,7 +26,7 @@ pub fn gen_winrt_upcall(sig: &MethodSignature, inner: TokenStream, gen: &Gen) ->
                     ::std::result::Result::Ok(ok__) => {
                         *result__ = ::std::mem::transmute_copy(&ok__);
                         ::std::mem::forget(ok__);
-                        ::windows::HRESULT(0)
+                        ::windows::runtime::HRESULT(0)
                     }
                     ::std::result::Result::Err(err) => err.into()
                 }
@@ -77,7 +77,7 @@ pub fn gen_winrt_abi(sig: &MethodSignature, gen: &Gen) -> TokenStream {
         }));
 
     quote! {
-        (this: ::windows::RawPtr, #(#params),*) -> ::windows::HRESULT
+        (this: ::windows::runtime::RawPtr, #(#params),*) -> ::windows::runtime::HRESULT
     }
 }
 
@@ -91,7 +91,7 @@ fn gen_winrt_invoke_arg(param: &MethodParam, gen: &Gen) -> TokenStream {
         if param.param.is_input() {
             quote! { ::std::slice::from_raw_parts(::std::mem::transmute_copy(&#name), #abi_size_name as _) }
         } else if param.signature.by_ref {
-            quote! { ::windows::ArrayProxy::from_raw_parts(::std::mem::transmute_copy(&#name), #abi_size_name).as_array() }
+            quote! { ::windows::runtime::ArrayProxy::from_raw_parts(::std::mem::transmute_copy(&#name), #abi_size_name).as_array() }
         } else {
             quote! { ::std::slice::from_raw_parts_mut(::std::mem::transmute_copy(&#name), #abi_size_name as _) }
         }
@@ -99,9 +99,9 @@ fn gen_winrt_invoke_arg(param: &MethodParam, gen: &Gen) -> TokenStream {
         if param.signature.kind.is_primitive() {
             quote! { #name }
         } else if param.signature.is_const {
-            quote! { &*(#name as *const <#kind as ::windows::Abi>::Abi as *const <#kind as ::windows::Abi>::DefaultType) }
+            quote! { &*(#name as *const <#kind as ::windows::runtime::Abi>::Abi as *const <#kind as ::windows::runtime::Abi>::DefaultType) }
         } else {
-            quote! { &*(&#name as *const <#kind as ::windows::Abi>::Abi as *const <#kind as ::windows::Abi>::DefaultType) }
+            quote! { &*(&#name as *const <#kind as ::windows::runtime::Abi>::Abi as *const <#kind as ::windows::runtime::Abi>::DefaultType) }
         }
     } else {
         quote! { ::std::mem::transmute_copy(&#name) }
@@ -118,11 +118,11 @@ pub fn gen_winrt_params(params: &[MethodParam], gen: &Gen) -> TokenStream {
 
             if param.signature.is_array {
                 if param.param.is_input() {
-                    quote! { #name: &[<#tokens as ::windows::Abi>::DefaultType], }
+                    quote! { #name: &[<#tokens as ::windows::runtime::Abi>::DefaultType], }
                 } else if param.signature.by_ref {
-                    quote! { #name: &mut ::windows::Array<#tokens>, }
+                    quote! { #name: &mut ::windows::runtime::Array<#tokens>, }
                 } else {
-                    quote! { #name: &mut [<#tokens as ::windows::Abi>::DefaultType], }
+                    quote! { #name: &mut [<#tokens as ::windows::runtime::Abi>::DefaultType], }
                 }
             } else if param.param.is_input() {
                 if param.is_convertible() {
@@ -134,7 +134,7 @@ pub fn gen_winrt_params(params: &[MethodParam], gen: &Gen) -> TokenStream {
             } else if param.signature.kind.is_nullable() {
                 quote! { #name: &mut ::std::option::Option<#tokens>, }
             } else if let ElementType::GenericParam(_) = param.signature.kind {
-                quote! { &mut <#tokens as ::windows::Abi>::DefaultType, }
+                quote! { &mut <#tokens as ::windows::runtime::Abi>::DefaultType, }
             } else if param.signature.pointers > 0 {
                 let tokens = gen_abi_sig(&param.signature, gen);
                 quote! { #name: #tokens, }
@@ -171,7 +171,7 @@ pub fn gen_winrt_method(
 
     let vtable_offset = Literal::u32_unsuffixed(method.vtable_offset);
     let constraints = gen_method_constraints(params, gen);
-    let args = params.iter().map(|p| gen_winrt_abi_arg(p));
+    let args = params.iter().map(gen_winrt_abi_arg);
     let params = gen_winrt_params(params, gen);
     let interface_name = gen_type_name(&interface.def, gen);
 
@@ -179,7 +179,7 @@ pub fn gen_winrt_method(
         let tokens = gen_name(&return_sig.kind, gen);
 
         if return_sig.is_array {
-            quote! { ::windows::Array<#tokens> }
+            quote! { ::windows::runtime::Array<#tokens> }
         } else {
             tokens
         }
@@ -190,7 +190,7 @@ pub fn gen_winrt_method(
     let return_arg = if let Some(return_sig) = &sig.return_sig {
         if return_sig.is_array {
             let return_sig = gen_name(&return_sig.kind, gen);
-            quote! { ::windows::Array::<#return_sig>::set_abi_len(&mut result__), &mut result__ as *mut _ as _ }
+            quote! { ::windows::runtime::Array::<#return_sig>::set_abi_len(&mut result__), &mut result__ as *mut _ as _ }
         } else {
             quote! { &mut result__ }
         }
@@ -202,7 +202,7 @@ pub fn gen_winrt_method(
     // arguments to ensure the call succeeds in the non-aggregating case.
     let composable_args = match interface.kind {
         InterfaceKind::Composable => quote! {
-            ::std::ptr::null_mut(), &mut ::std::option::Option::<::windows::IInspectable>::None as *mut _ as _,
+            ::std::ptr::null_mut(), &mut ::std::option::Option::<::windows::runtime::IInspectable>::None as *mut _ as _,
         },
         InterfaceKind::Extend => quote! {
             ::std::mem::transmute_copy(&derived__), base__ as *mut _ as _,
@@ -214,22 +214,28 @@ pub fn gen_winrt_method(
         if return_sig.is_array {
             quote! {
                 let mut result__: #return_type_tokens = ::std::mem::zeroed();
-                (::windows::Interface::vtable(this).#vtable_offset)(::std::mem::transmute_copy(this), #(#args,)* #composable_args #return_arg)
+                (::windows::runtime::Interface::vtable(this).#vtable_offset)(::std::mem::transmute_copy(this), #(#args,)* #composable_args #return_arg)
                     .and_then(|| result__ )
             }
         } else {
+            let abi_type_name = gen_abi_type_name(&return_sig.kind, gen);
+
             quote! {
-                let mut result__: <#return_type_tokens as ::windows::Abi>::Abi = ::std::mem::zeroed();
-                    (::windows::Interface::vtable(this).#vtable_offset)(::std::mem::transmute_copy(this), #(#args,)* #composable_args #return_arg)
+                let mut result__: #abi_type_name = ::std::mem::zeroed();
+                    (::windows::runtime::Interface::vtable(this).#vtable_offset)(::std::mem::transmute_copy(this), #(#args,)* #composable_args #return_arg)
                         .from_abi::<#return_type_tokens>(result__ )
             }
         }
     } else {
         quote! {
-            (::windows::Interface::vtable(this).#vtable_offset)(::std::mem::transmute_copy(this), #(#args,)* #composable_args).ok()
+            (::windows::runtime::Interface::vtable(this).#vtable_offset)(::std::mem::transmute_copy(this), #(#args,)* #composable_args).ok()
         }
     };
 
+    // TODO: need to consolidate this cfg generation so we
+
+    let features = interface_method_features(&interface.def, sig, gen);
+
     let deprecated = if method.is_deprecated {
         quote! { #[cfg(feature = "deprecated")] }
     } else {
@@ -239,7 +245,8 @@ pub fn gen_winrt_method(
     match interface.kind {
         InterfaceKind::Default => quote! {
             #deprecated
-            pub fn #name<#constraints>(&self, #params) -> ::windows::Result<#return_type_tokens> {
+            #features
+            pub fn #name<#constraints>(&self, #params) -> ::windows::runtime::Result<#return_type_tokens> {
                 let this = self;
                 unsafe {
                     #vcall
@@ -249,8 +256,9 @@ pub fn gen_winrt_method(
         InterfaceKind::NonDefault | InterfaceKind::Overridable => {
             quote! {
                 #deprecated
-                pub fn #name<#constraints>(&self, #params) -> ::windows::Result<#return_type_tokens> {
-                    let this = &::windows::Interface::cast::<#interface_name>(self)?;
+                #features
+                pub fn #name<#constraints>(&self, #params) -> ::windows::runtime::Result<#return_type_tokens> {
+                    let this = &::windows::runtime::Interface::cast::<#interface_name>(self)?;
                     unsafe {
                         #vcall
                     }
@@ -260,7 +268,8 @@ pub fn gen_winrt_method(
         InterfaceKind::Static | InterfaceKind::Composable => {
             quote! {
                 #deprecated
-                pub fn #name<#constraints>(#params) -> ::windows::Result<#return_type_tokens> {
+                #features
+                pub fn #name<#constraints>(#params) -> ::windows::runtime::Result<#return_type_tokens> {
                     Self::#interface_name(|this| unsafe { #vcall })
                 }
             }
@@ -268,9 +277,11 @@ pub fn gen_winrt_method(
         InterfaceKind::Extend => {
             let interface_name = to_ident(interface.def.name());
             quote! {
-                pub fn #name<#constraints>(self, #params) -> ::windows::Result<#return_type_tokens> {
+                // TODO: why no deprecated?
+                #features
+                pub fn #name<#constraints>(self, #params) -> ::windows::runtime::Result<#return_type_tokens> {
                     unsafe {
-                        let (derived__, base__) = ::windows::Compose::compose(self);
+                        let (derived__, base__) = ::windows::runtime::Compose::compose(self);
                         #return_type_tokens::#interface_name(|this| unsafe { #vcall })
                     }
                 }
@@ -318,15 +329,15 @@ pub fn gen_winrt_produce_type(param: &MethodParam, gen: &Gen) -> TokenStream {
 
     if param.signature.is_array {
         if param.param.is_input() {
-            quote! { &[<#tokens as ::windows::Abi>::DefaultType] }
+            quote! { &[<#tokens as ::windows::runtime::Abi>::DefaultType] }
         } else if param.signature.by_ref {
-            quote! { &mut ::windows::Array<#tokens> }
+            quote! { &mut ::windows::runtime::Array<#tokens> }
         } else {
-            quote! { &mut [<#tokens as ::windows::Abi>::DefaultType] }
+            quote! { &mut [<#tokens as ::windows::runtime::Abi>::DefaultType] }
         }
     } else if param.param.is_input() {
         if let ElementType::GenericParam(_) = param.signature.kind {
-            quote! { &<#tokens as ::windows::Abi>::DefaultType }
+            quote! { &<#tokens as ::windows::runtime::Abi>::DefaultType }
         } else if param.signature.kind.is_primitive() {
             quote! { #tokens }
         } else if param.signature.kind.is_nullable() {
@@ -343,7 +354,7 @@ pub fn gen_winrt_produce_type(param: &MethodParam, gen: &Gen) -> TokenStream {
 
 pub fn gen_phantoms(def: &TypeDef) -> impl Iterator<Item = TokenStream> + '_ {
     def.generics.iter().map(move |g| {
-        let g = gen_name(g, &Gen::Absolute);
+        let g = gen_name(g, &Gen::absolute());
         quote! { ::std::marker::PhantomData::<#g> }
     })
 }
@@ -352,8 +363,8 @@ pub fn gen_constraints(def: &TypeDef) -> TokenStream {
     def.generics
         .iter()
         .map(|g| {
-            let g = gen_name(g, &Gen::Absolute);
-            quote! { #g: ::windows::RuntimeType + 'static, }
+            let g = gen_name(g, &Gen::absolute());
+            quote! { #g: ::windows::runtime::RuntimeType + 'static, }
         })
         .collect()
 }
